#!/bin/sh

createRedHatScript() {
	# Red Hat heredoc start
	(
		cat <<-EOF
		#!/bin/sh
	
		#chkconfig: 2345 99 01
		#description:  Optim Proxy 
	
		# Source in function library
		. /etc/rc.d/init.d/functions

		PROXYDIR=$OPTIM_LOCATION/proxy
		if [ ! -x \$PROXYDIR/eclipse ]
		then
			ERROR="There is no proxy at specified location"
			failure optimproxy \$ERROR 
   		 	exit 5
		fi

		LD_LIBRARY_PATH=$OPTIM_LOCATION/shared/bin
		export LD_LIBRARY_PATH
	
		# Log dir exists?
		LOGDIR=\$PROXYDIR/log
		LOGFILE=\$LOGDIR/proxy.log
		if [ ! -d \$LOGDIR ] 
		then
			mkdir \$LOGDIR
			chown $USER \$LOGDIR
		fi
		PID_FILE=/var/run/optimproxy.pid
	
		# Shell functions
		_start() {
			if [ ! -f \$PID_FILE ]
			then
				(
				cat <<-DAEMON_HEREDOC
					\$PROXYDIR/eclipse 1>\$LOGFILE 2>&1 &
					echo \\\$! >/tmp/optimproxy.pid
				DAEMON_HEREDOC
				) > /tmp/\$$
				chmod +x /tmp/\$$
				echo -n Starting Optim proxy
				daemon --user $USER /tmp/\$$
				if [ \$? -eq 0 ]
				then
					rm -f /tmp/\$$
					cp /tmp/optimproxy.pid \$PID_FILE
					chown $USER \$PID_FILE
				fi
				echo
				rm -f /tmp/\$$
				chown $USER \$LOGFILE
			fi
		}

		_stop() {
			if [ -s \$PID_FILE ]
			then
				echo -n Stopping Optim proxy
				PID=\`cat \$PID_FILE\`
				killproc -p \$PID_FILE eclipse -9
				RC=\$?
				if [ \$RC -eq 0 ]
				then
					rm -f \$PID_FILE
				fi	
				echo
			fi
		}

		case "\$1" in
			"start")
				_start
				;;
			"stop")
				_stop
				;;
			"restart")
				_stop
				_start
				;;
		esac
		exit 0 
	EOF
	) > $OUTFILE
}

createUbuntuScript() {
	# Ubuntu heredoc start
	(
		cat <<-EOF
		#!/bin/sh

		### BEGIN INIT INFO
		# Provides:		optimproxy
		# Required-Start:	$remote_fs $syslog
		# Required-Stop:	$remote_fs $syslog
		# Default-Start:	2 3 4 5
		# Default-Stop:	 0 1 6
		# Short-Description: Start Optim Proxy as daemon at boot time
		# Description:	  Enable service provided by optimproxy daemon.
		### END INIT INFO

		PROXYDIR=$OPTIM_LOCATION/proxy
		if [ ! -x \$PROXYDIR/eclipse ]
		then
			echo "Could not find Optim Proxy" >&2
			exit 5
		fi
	
		LD_LIBRARY_PATH=$OPTIM_LOCATION/shared/bin
		export LD_LIBRARY_PATH
	
		# Log dir exists?
		LOGDIR=\$PROXYDIR/log
		LOGFILE=\$LOGDIR/proxy.log
		if [ ! -d \$LOGDIR ] 
		then
			mkdir \$LOGDIR
			chown $USER \$LOGDIR
		fi
		PID_FILE=/var/run/optimproxy.pid

		# Shell functions
		_start() {
			if [ ! -f \$PID_FILE ]
			then
				(
				cat <<-DAEMON_HEREDOC
					#!/bin/sh
					\$PROXYDIR/eclipse 1>\$LOGFILE 2>&1 &
					echo \\\$! >/tmp/optimproxy.pid
				DAEMON_HEREDOC
				) >/tmp/\$$
				chmod +x /tmp/\$$
				echo -n Starting Optim proxy
				start-stop-daemon --start --chuid $USER --chdir \$PROXYDIR --pidfile /tmp/$$.fakeid --startas /tmp/\$$
				if [ \$? -eq 0 ]
				then
					cp /tmp/optimproxy.pid \$PID_FILE
					chown $USER \$PID_FILE
				fi
#				rm -f /tmp/\$$
				chown $USER \$LOGFILE
			fi
		}

		_stop() {
			if [ -s \$PID_FILE ]
			then
				echo -n Stopping Optim proxy
				PID=\`cat \$PID_FILE\`
				kill -9 \$PID
				RC=\$?
				if [ \$RC -eq 0 ]
				then
					rm -f \$PID_FILE
				fi
				echo
			fi
		}

		case "\$1" in
		"start")
			_start
			;;
		"stop")
			_stop
			;;
		"restart")
			_stop
			_start
			;;
	esac

	EOF
	) > $OUTFILE
}

createAIXScript() {
	# AIX heredoc start
	(
		cat <<-EOF
		#!/bin/sh

		PROXYDIR=$OPTIM_LOCATION/proxy
		if [ ! -x \$PROXYDIR/eclipse ]
		then
			echo "Could not find Optim Proxy" >&2
			exit 5
		fi
	
		LD_LIBRARY_PATH=$OPTIM_LOCATION/shared/bin
		export LD_LIBRARY_PATH
	
		# Log dir exists?
		LOGDIR=\$PROXYDIR/log
		LOGFILE=\$LOGDIR/proxy.log
		if [ ! -d \$LOGDIR ] 
		then
			mkdir \$LOGDIR
			chown $USER \$LOGDIR
		fi
		PID_FILE=/var/run/optimproxy.pid

		# Shell functions
		_start() {
			if [ ! -f \$PID_FILE ]
			then
				(
				cat <<-DAEMON_HEREDOC
					#!/bin/sh
					\$PROXYDIR/eclipse 1>\$LOGFILE 2>&1 &
					echo \\\$! >/tmp/optimproxy.pid
				DAEMON_HEREDOC
				) >/tmp/\$$
				chmod +x /tmp/\$$
				echo "Starting Optim proxy\c"
				su $USER -c /tmp/\$$
				if [ \$? -eq 0 ]
				then
					cp /tmp/optimproxy.pid \$PID_FILE
					chown $USER \$PID_FILE
				fi
#				rm -f /tmp/\$$
				chown $USER \$LOGFILE
			fi
		}

		_stop() {
			if [ -s \$PID_FILE ]
			then
				echo "Stopping Optim proxy\c"
				PID=\`cat \$PID_FILE\`
				kill -9 \$PID
				RC=\$?
				if [ \$RC -eq 0 ]
				then
					rm -f \$PID_FILE
				fi
			fi
		}

		case "\$1" in
		"start")
			_start
			;;
		"stop")
			_stop
			;;
		"restart")
			_stop
			_start
			;;
	esac

	EOF
	) > $OUTFILE
}

createSunOSScript() {
	# SunOS heredoc start
	(
		cat <<-EOF
		#!/bin/sh

		PROXYDIR=$OPTIM_LOCATION/proxy
		if [ ! -x \$PROXYDIR/eclipse ]
		then
			echo "Could not find Optim Proxy" >&2
			exit 5
		fi
	
		LD_LIBRARY_PATH=$OPTIM_LOCATION/shared/bin
		export LD_LIBRARY_PATH
	
		# Log dir exists?
		LOGDIR=\$PROXYDIR/log
		LOGFILE=\$LOGDIR/proxy.log
		if [ ! -d \$LOGDIR ] 
		then
			mkdir \$LOGDIR
			chown $USER \$LOGDIR
		fi
		PID_FILE=/var/run/optimproxy.pid

		# Shell functions
		_start() {
			if [ ! -f \$PID_FILE ]
			then
				(
				cat <<-DAEMON_HEREDOC
					#!/bin/sh
					\$PROXYDIR/eclipse 1>\$LOGFILE 2>&1 &
					echo \\\$! >/tmp/optimproxy.pid
				DAEMON_HEREDOC
				) >/tmp/\$$
				chmod +x /tmp/\$$
				echo "Starting Optim proxy\c"
				su $USER -c /tmp/\$$
				if [ \$? -eq 0 ]
				then
					cp /tmp/optimproxy.pid \$PID_FILE
					chown $USER \$PID_FILE
				fi
#				rm -f /tmp/\$$
				chown $USER \$LOGFILE
			fi
		}

		_stop() {
			if [ -s \$PID_FILE ]
			then
				echo "Stopping Optim proxy\c"
				PID=\`cat \$PID_FILE\`
				kill -9 \$PID
				RC=\$?
				if [ \$RC -eq 0 ]
				then
					rm -f \$PID_FILE
				fi
			fi
		}

		case "\$1" in
		"start")
			_start
			;;
		"stop")
			_stop
			;;
		"restart")
			_stop
			_start
			;;
	esac

	EOF
	) > $OUTFILE
}

createHPUXScript() {
	# HPUX heredoc start
	(
		cat <<-EOF
		#!/bin/sh
		#
		# @(#)B.11.31_LR 
		#
		# NOTE:    This script is not configurable!  Any changes made to this
		#          script will be overwritten when you upgrade to the next
		#          release of HP-UX.
		#
		# WARNING: Changing this script in any way may lead to a system that
		#          is unbootable.  Do not modify this script.
		
		#
		# This is the init script for starting and stopping the Optim proxy service.
		#
		
		# Allowed exit values:
		#	0 = success; causes "OK" to show up in checklist.
		#	1 = failure; causes "FAIL" to show up in checklist.
		#	2 = skip; causes "N/A" to show up in the checklist.
		#           Use this value if execution of this script is overridden
		#	    by the use of a control variable, or if this script is not
		#	    appropriate to execute for some other reason.
		#       3 = reboot; causes the system to be rebooted after execution.
		#	4 = background; causes "BG" to show up in the checklist.
		#           Use this value if this script starts a process in background mode.
		
		# Input and output:
		#	stdin is redirected from /dev/null
		#
		#	stdout and stderr are redirected to the /etc/rc.log file
		#	during checklist mode, or to the console in raw mode.
		
		PATH=/usr/sbin:/usr/bin:/sbin
		export PATH
		
		# NOTE: If your script executes in run state 0 or state 1, then /usr might
		#	not be available.  Do not attempt to access commands or files in
		#	/usr unless your script executes in run state 2 or greater.  Other
		#	file systems typically not mounted until run state 2 include /var
		#	and /opt.
		
		rval=0
		
		# Start: Optim proxy specific environment setup
		PROXYDIR=$OPTIM_LOCATION/proxy
		if [ ! -x \$PROXYDIR/eclipse ]
		then
			echo "Could not find Optim Proxy" >&2
			exit 5
		fi
	
		LD_LIBRARY_PATH=$OPTIM_LOCATION/shared/bin
		export LD_LIBRARY_PATH
	
		# Log dir exists?
		LOGDIR=\$PROXYDIR/log
		LOGFILE=\$LOGDIR/proxy.log
		if [ ! -d \$LOGDIR ] 
		then
			mkdir \$LOGDIR
			chown $USER \$LOGDIR
		fi
		PID_FILE=/var/run/optimproxy.pid
		# End: Optim proxy specific environment setup

		# Check the exit value of a command run by this script.  If non-zero, the
		# exit code is echoed to the log file and the return value of this script
		# is set to indicate failure.
		
		set_return() {
			x=\$?
			if [ \$x -ne 0 ]; then
				echo "EXIT CODE: \$x"
				rval=1	# script FAILed
			fi
		}
		
		# Kill the named process(es).
		# $1=<search pattern for your process>
		
		killproc() {
			pid=\`ps -el | awk '( (\$NF ~ /'"\$1"'/) && (\$4 != mypid) && (\$5 != mypid)  ){ print \$4 }' mypid=\$$ \`
			if [ "X\$pid" != "X" ]; then
				if kill "$pid"; then
					echo "\$1 stopped"
				else
					rval=1
					echo "Unable to stop \$1"
				fi
			fi
		}
		

		# Start: Optim proxy specific shell functions
		_start() {
			if [ ! -f \$PID_FILE ]
			then
				(
				cat <<-DAEMON_HEREDOC
					#!/bin/sh
					\$PROXYDIR/eclipse 1>\$LOGFILE 2>&1 &
					echo \\\$! >/tmp/optimproxy.pid
				DAEMON_HEREDOC
				) >/tmp/\$$
				chmod +x /tmp/\$$
				echo "Starting Optim proxy\c"
				su $USER -c /tmp/\$$
				if [ \$? -eq 0 ]
				then
					cp /tmp/optimproxy.pid \$PID_FILE
					chown $USER \$PID_FILE
				fi
#				rm -f /tmp/\$$
				chown $USER \$LOGFILE
			fi
		}

		_stop() {
			if [ -s \$PID_FILE ]
			then
				echo "Stopping Optim proxy\c"
				PPID=\`cat \$PID_FILE\`
				# Kill proxy and child processes, if any 
				for PID in \`ps -ef | awk '\$3 == '\${PPID}' { print \$2 }'\` 
				do
					kill -TERM \$PID
				done	
				# Kill parent process
				kill -TERM \$PPID
				RC=\$?
				if [ \$RC -eq 0 ]
				then
					rm -f \$PID_FILE
				fi
			fi
		}
		# End: Optim proxy specific shell functions
		
		case \$1 in
		'start_msg')
			# Emit a _short_ message relating to running this script with
			# the "start" argument; this message appears as part of the checklist.
			echo "Starting the Optim proxy subsystem"
			;;
		
		'stop_msg')
			# Emit a _short_ message relating to running this script with
			# the "stop" argument; this message appears as part of the checklist.
			echo "Stopping the Optim proxy subsystem"
			;;
		
		'start')
		
			# source the system configuration variables
			if [ -f /etc/rc.config ] ; then
				. /etc/rc.config
			else
				echo "ERROR: /etc/rc.config defaults file MISSING"
			fi
		
			# Check to see if this script is allowed to run by checking
			# the control variable OPTIM_PROXY that is located in the
			# /etc/rc.config.d/optimproxy file.  If that file has not been
			# created and does not set the OPTIM_PROXY varaible to 1, we
			# we will not start the daemon.
			if [ "\$OPTIM_PROXY" != 1 ]; then
				rval=2
			else
				# Execute the commands to start your subsystem
				_start
			fi
			;;
		
		'stop')
			# source the system configuration variables
			if [ -f /etc/rc.config ] ; then
				. /etc/rc.config
			else
				echo "ERROR: /etc/rc.config defaults file MISSING"
			fi
		
			# Check to see if this script is allowed to run...
			if [ "\$OPTIM_PROXY" != 1 ]; then
				rval=2
			else
				# Execute the commands to stop your subsystem
				_stop
			fi
			;;
		
		*)
			echo "usage: $0 {start|stop|start_msg|stop_msg}"
			rval=1
			;;
		esac
		
		exit $rval

	EOF
	) > $OUTFILE
}

TYPE=`uname`

case "$TYPE" in 
	"Linux")
		# Determine Linux flavor
		FLAVOR=`lsb_release -d | awk '{ print $2 }'`
		;;
	*)
		FLAVOR=$TYPE
		;;
esac

# Name of the init file to generate
OUTFILE=optimproxy

# Ensure the script is executed from Optim proxy directory and
# set location for use in setting Optim location in init script
HERE=`pwd`
cd ..
OPTIM_LOCATION=`pwd`
BASENAME=`basename $HERE`
if [ "$BASENAME" != "proxy" ]
then
	   echo "ERROR: You must run this script from the Optim proxy directory" >&2
	   exit 1
fi
# Determine the user account for the daemon to start service as
USER=`ls -ld | awk '{ print $3 }'`
cd $HERE

if [ "$FLAVOR" = "Red" ]
then
	createRedHatScript
elif [ "$FLAVOR" = "Ubuntu" ]
then
	createUbuntuScript
elif [ "$FLAVOR" = "AIX" ]
then
	createAIXScript
elif [ "$FLAVOR" = "HP-UX" ]
then
	createHPUXScript
elif [ "$FLAVOR" = "SunOS" ]
then
	createSunOSScript
fi
chmod +x $OUTFILE
